@@ -82,16 +82,22 @@ async function main() {
     'language/expressions/await/await-in-nested-generator.js',
     'language/expressions/class/class-name-ident-await-escaped.js',
     'language/expressions/class/class-name-ident-await.js',
+    'language/expressions/class/static-init-await-reference.js',
+    'language/expressions/dynamic-import/2nd-param-await-ident.js',
     'language/expressions/dynamic-import/assignment-expression/await-identifier.js',
+    'language/expressions/function/static-init-await-reference.js',
+    'language/expressions/generators/static-init-await-reference.js',
+    'language/expressions/in/private-field-rhs-await-absent.js',
     'language/expressions/object/identifier-shorthand-await-strict-mode.js',
+    'language/expressions/object/method-definition/static-init-await-reference-accessor.js',
+    'language/expressions/object/method-definition/static-init-await-reference-generator.js',
+    'language/expressions/object/method-definition/static-init-await-reference-normal.js',
     'language/module-code/top-level-await/new-await-script-code.js',
     'language/reserved-words/await-script.js',
     'language/statements/class/class-name-ident-await-escaped.js',
     'language/statements/class/class-name-ident-await.js',
     'language/statements/labeled/value-await-non-module-escaped.js',
     'language/statements/labeled/value-await-non-module.js',
-    'language/expressions/dynamic-import/2nd-param-await-ident.js',
-    'language/expressions/in/private-field-rhs-await-absent.js',
 
     // Skip these tests because we don't currently validate the contents of
     // regular expressions. We could do this but it's not necessary to parse
@@ -142,6 +148,8 @@ async function main() {
 
     if (yaml.features) {
       if (yaml.features.includes('hashbang')) return
+      if (yaml.features.includes('decorators')) return
+      if (yaml.features.includes('regexp-v-flag')) return
       if (yaml.features.includes('regexp-match-indices')) return
       if (yaml.features.includes('regexp-named-groups')) return
       if (yaml.features.includes('regexp-unicode-property-escapes')) return
